class SonaApiError(Exception):
    def __init__(self, message, api_response): #, errors

        # Call the base class constructor with the parameters it needs
        super().__init__(message)

        #TODO more debugging info of failed http api request
        self.api_response = api_response
        # self.errors = errors



from marshmallow import Schema,fields, pre_load

class ResearcherSchema(Schema):
    email = fields.Email()
    first_name = fields.Str()
    full_name = fields.Str()
    last_name = fields.Str()
    office = fields.Str()
    phone = fields.Str()  #TODO phone field

class ExperimentRefSchema(Schema):
    exp_name = fields.Str()
    experiment_id = fields.Number()
    qualifies = fields.Bool()
    qualifies_text = fields.Str()


class StudyInfoSchema(Schema):
    course_credits = fields.List(fields.Str())
    PI = fields.Str(allow_none=True) #was None TODO
    abstract_text = fields.Str()
    credits_label = fields.Str()
    credits_part1_value = fields.Float(required=False, allow_none=True)
    credits_part2_value = fields.Float(required=False, allow_none=True)
    credits_part3_value = fields.Float(required=False, allow_none=True)
    credits_part4_value = fields.Float(required=False, allow_none=True) #ALL THESE IN VPh part floats
    credits_total_value = fields.Float(required=True)
    description_text = fields.Str() #TODO html code field
    display_timeslots_button = fields.Bool()
    display_timeslots_reason = fields.Str()
    disqualifier_info = fields.Nested(ExperimentRefSchema(many=True))
    disqualifier_text = fields.Str()
    duration_part1 = fields.Float(allow_none=True)
    duration_part2 = fields.Float(allow_none=True) #TODO maybe number not float cause full minutes always?
    duration_part3 = fields.Float(allow_none=True) #all this in minutes number
    duration_part4 = fields.Float(allow_none=True)
    duration_total = fields.Float(required=True) # was 45 Minuten needs to be preprocessed,
    eligibility_reqs = fields.Str(allow_none=True) #was None TODO
    exp_name = fields.Str()
    experiment_id = fields.Number()
    invitation_code_text = fields.Str()
    multipart_count = fields.Number()
    preparation = fields.Str()
    prereq_info = fields.Nested(ExperimentRefSchema(many=True))
    prereq_text = fields.Str()
    researchers = fields.Nested(ResearcherSchema(many=True))
    twopart_study_text = fields.Str()
    website_label = fields.Str()
    website_link_label = fields.Str()
    website_link_text = fields.Str()

    @pre_load
    def duration_credits_float(self, in_data, **kwargs):
        # Credits get float value from "3,5 VPh XXXX" string
        in_data['credits_part1_value'] = float(in_data['credits_part1_value'].replace(" VPh", "")) if in_data['credits_part1_value'] else None
        in_data['credits_part2_value'] = float(in_data['credits_part2_value'].replace(" VPh", "")) if in_data['credits_part2_value'] else None
        in_data['credits_part3_value'] = float(in_data['credits_part3_value'].replace(" VPh", "")) if in_data['credits_part3_value'] else None
        in_data['credits_part4_value'] = float(in_data['credits_part4_value'].replace(" VPh", "")) if in_data['credits_part4_value'] else None
        in_data['credits_total_value'] = float(in_data['credits_total_value'].replace(" VPh", ""))

        #Duration get minutes from " 45 Min XXXX" string
        in_data['duration_part1'] = float(in_data['duration_part1'].strip().split(" ")[0]) if in_data['duration_part1'] else None
        in_data['duration_part2'] = float(in_data['duration_part2'].strip().split(" ")[0]) if in_data['duration_part2'] else None
        in_data['duration_part3'] = float(in_data['duration_part3'].strip().split(" ")[0]) if in_data['duration_part3'] else None
        in_data['duration_part4'] = float(in_data['duration_part4'].strip().split(" ")[0]) if in_data['duration_part4'] else None
        in_data['duration_total'] = float(in_data['duration_total'].strip().split(" ")[0])
        return in_data


from marshmallow import Schema,fields


class StudySchema(Schema):
    #TODO create html field for these like exp name which contain html
    exp_name = fields.Str()
    experiment_id = fields.Int()
    first_line = fields.Str()
    second_line = fields.Str()
    timeslots_available = fields.Bool()
    web_flag = fields.Bool()


class StudyPageSchema(Schema):
    # study_signups = fields.List(fields.Str())
    #TODO what with this
    # study_signups = fields.Nested(StudySchema(many=True))
    application_text = fields.Str()
    studies = fields.Nested(StudySchema(many=True))


import requests
import shutil
from pprint import pprint

#TODO package with top level here for import
from SonaWrap.schemas.Schedule import ScheduleSchema
from SonaWrap.schemas.MainMenuInfo import MainMenuInfoSchema
from SonaWrap.schemas.StudyPage import StudyPageSchema
from SonaWrap.schemas.StudyInfo import StudyInfoSchema

from SonaWrap.SonaApiError import SonaApiError

class SonaWrap:
    def __init__(self, token:str=None, username:str = None, password:str = None, base_host:str="https://psywue.sona-systems.com"):
        self._host = base_host
        self._token = token if token else self._authenticate(username,password)
        print("Initialized SonaWrap with token "+ self._token)

    def get_token(self):
        return self._token

    def _request_invalidator(self, r):
        if r.status_code != 200 or r.json()["ErrorCode"] != 0:
            print(r.text)
            print(r.json()["Errors"])
            print(r.json())
            #TODO own exception here
            raise SonaApiError("Request to SoNA api failed", r.json())
        return r.json()["Result"]

    def test_connection(self):
        #TODO export cause unused
        url = self._host + "/services/SonaMobileAPI.svc/TestConnection"
        r = requests.post(url)
        # r_json = r.json()

        # print(r.text)
        #TODO schema
        #TODO better way to invalidate errored
        r = self._request_invalidator(r)
        return r
        #TODO return type

    def _login_page(self):
        #TODO export cause unused
        url = self._host + "/services/SonaMobileAPI.svc/GetLoginPageInfoV2"
        r = requests.post(url)
        r = self._request_invalidator(r)
        #TODO schema
        # print(r.text)
        return r
        #TODO return type


    def custom_icon(self):
        url = self._host + "/custom/customlogo.png"

        r = requests.get(url,stream=True)
        # print(r.status_code)
        with open('logo.png', 'wb') as out_file:
            shutil.copyfileobj(r.raw, out_file)
        del r
        #TODO return image here as object instead of saving at all

    def _authenticate(self, username:str, password:str) -> str: #TODO return type token given here
        # r_login_page = self._login_page()

        url = self._host + "/services/SonaMobileAPI.svc/AuthenticateV2"

        body = {
            "p_username": username,
            "p_password": password,
            "p_language_pref":"DE",
            "p_mobile_version":"2.8.8"
        }

        print("Authenticasting")
        r = requests.post(url, json=body)

        #TODO own authorize exception
        token = self._request_invalidator(r)['sona_auth']

        # print(r.text)
        return token

    #TODO nextLogin Page request but no useful info in capture

    def my_schedule(self):
        url = self._host + "/services/SonaMobileAPI.svc/GetMyScheduleInfo"

        body = {
            "p_sessionToken": self._token
        }

        r = requests.post(url, json=body)
        result = self._request_invalidator(r)

        schema = ScheduleSchema()
        resultObj = schema.load(result)

        return resultObj


    def main_menu_info(self):
        url = self._host + "/services/SonaMobileAPI.svc/GetMainMenuInfo"

        body = {
            "p_sessionToken": self._token
        }

        r = requests.post(url, json=body)
        result = self._request_invalidator(r)

        schema = MainMenuInfoSchema()
        resultObj = schema.load(result)

        return resultObj


    def study_page_info(self) -> StudyPageSchema:
        url = self._host + "/services/SonaMobileAPI.svc/GetStudiesPageInfo"

        body = {
            "p_sessionToken": self._token
        }

        r = requests.post(url, json=body)
        result = self._request_invalidator(r)

        schema = StudyPageSchema()
        resultObj = schema.load(result)

        return resultObj

    def study_info(self, experiment_id:int) -> StudyInfoSchema:
        url = self._host + "/services/SonaMobileAPI.svc/GetStudyInfo"

        body = {
            "p_sessionToken": self._token,
            "p_experiment_id": experiment_id
        }

        r = requests.post(url, json=body)
        result = self._request_invalidator(r)

        schema = StudyInfoSchema()
        resultObj = schema.load(result)

        return resultObj


from marshmallow import Schema,fields, pre_load

class ScheduleSchema(Schema):
    course_credits = fields.List(fields.Str())
    display_course_credits = fields.Bool()
    display_overall_credits = fields.Bool()
    overall_credits_earned = fields.Float()
    overall_credits_needed = fields.Float()
    study_signups = fields.List(fields.Str())
    text_progress_display = fields.Str()

    @pre_load
    def credits_floatification(self, in_data, **kwargs):
        # converts cause real content is stupid
        in_data["overall_credits_earned"] = in_data["overall_credits_earned"].split(":")[1]
        in_data["overall_credits_needed"] = in_data["overall_credits_needed"].split(":")[1]
        return in_data




from marshmallow import Schema,fields

class MainMenuInfoSchema(Schema):
    announcement_display = fields.Bool()
    announcement_name = fields.Str()
    announcement_text = fields.Str()
    credits_display = fields.Bool()
    credits_earned = fields.Float()
    credits_name = fields.Str()
    credits_needed = fields.Float()
    credits_pending = fields.Float()
    email_questions_text = fields.Str()
    faq_enabled = fields.Bool()
    logo_url = fields.Url()
    #TODO what this cause for now was null
    # signups = fields.List(field)
    signups_display = fields.Bool()
    signups_name = fields.Str()
    site_name = fields.Str()
    username_name = fields.Str()
    username_text = fields.Str()
    username_type = fields.Str()